#include <config.h>
#include "auth.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "msg.h"
#include "intl.h"
#include "klist.h"
#include "list.h"
#include "cache.h"
#include "web.h"
#include "interface.h"
#include "nav.h"
#include "history.h"
#include "misc.h"

void Auth_confirmed(DilloDoc *dd);
void Auth_refused(DilloDoc *dd);
void Auth_load_authenticated(DilloDoc *dd, DilloUrl *url);

typedef struct _Realm Realm;

struct _Realm {
   DilloUrl *base_url;
   GString *auth;
};

static Realm *realms = NULL;
static gint num_realms;
static gint realms_max = 16;

void a_Auth_byrealm(GString *auth_realm, DilloUrl *NewUrl, DilloDoc *dd)
{
   /* we already have a password dialogue open */
   if (dd->auth_await_url)
      return;

   dd->auth_await_url = a_Url_dup(NewUrl);
   auth_realm = g_string_append(auth_realm, "\n");
   a_Interface_passwd_dialog(dd->bw, auth_realm->str,
                             Auth_confirmed, dd,
                             Auth_refused, dd);
}

GString *a_Auth_byurl(DilloUrl *n)
{
   gchar *offset;
   int i, longest = -1, len = 0, longlen = 0;
   gchar *ptr;

   if (!n)
      return NULL;

   for (i = 0; i < num_realms; i++) {
      ptr = URL_STR(realms[i].base_url);
      offset = strrchr(ptr, '/');
      if (!offset)
         offset = ptr + strlen(ptr);
      if (strncmp(URL_STR(n), ptr, (char*) offset - (char*) ptr) == 0) {
         len = (gchar *) offset - (gchar *) ptr;
         if (longlen <= len) {
            longlen=len;
            longest=i;
         }
      }
   }
   return longest == -1 ? NULL : realms[longest].auth;
}

void Auth_confirmed(DilloDoc *dd)
{
   DilloUrl *NewUrl;
   GString *new_auth;
   GtkEntry *uentry,*pentry;
   GString *up = g_string_new("");
   gchar *encoded;

   if (!dd->auth_await_url)
      return;

   NewUrl = dd->auth_await_url;

   /* are we still trying to browse the url that wants authentication? */

   if (a_Nav_stack_size(dd) != 0
         && a_Url_cmp(a_History_get_url(NAV_TOP(dd)), NewUrl) != 0) {
      MSG(_("Browser Url has changed\n"));
      /* free the url here, maybe ? */
      a_Url_free(NewUrl);
      dd->auth_await_url = NULL;
      return;
   }

   /* fetch and encode authorization */
   uentry = GTK_ENTRY(dd->bw->passwd_dialog_uentry);
   pentry = GTK_ENTRY(dd->bw->passwd_dialog_pentry);
   /* ... */
   g_string_sprintfa(up, "%s:%s", gtk_entry_get_text(uentry),
                     gtk_entry_get_text(pentry));
   new_auth = g_string_new("");
   encoded = a_Misc_encode_base64(up->str);
   g_string_sprintfa(new_auth, "Basic %s", encoded);
   g_free(encoded);
   a_List_add(realms, num_realms, realms_max);
   realms[num_realms].base_url = a_Url_dup(NewUrl);
   realms[num_realms].auth = new_auth;
   num_realms++;
   Auth_load_authenticated(dd, NewUrl);
   dd->auth_await_url = NULL;
}

void Auth_refused(DilloDoc *dd)
{
   DilloUrl *NewUrl;

   if (!dd->auth_await_url)
      return;
   NewUrl = dd->auth_await_url;
   g_return_if_fail(NewUrl);
   a_Url_free(NewUrl);
   dd->auth_await_url = NULL;
}

void Auth_load_authenticated(DilloDoc *dd, DilloUrl *NewUrl)
{
   a_Url_set_flags(NewUrl, URL_FLAGS(NewUrl) | URL_E2EReload | URL_RealmAccess);
   a_Nav_push(dd, NewUrl);
   a_Url_free(NewUrl);
}
  
void a_Auth_freeall(void)
{
}

