/*
 * File: commands.c
 *
 * Copyright (C) 1997 Raph Levien <raph@acm.org>
 * Copyright (C) 1999 Sammy Mannaert <nstalkie@tvd.be>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */

#include <config.h>
#include <gtk/gtk.h>
#include <stdio.h>              /* for sprintf */
#include <sys/time.h>           /* for gettimeofday (testing gorp only) */
#include <unistd.h>
#include <string.h>             /* for strcat() */

#include "intl.h"
#include "msg.h"
#include "bookmark.h"
#include "interface.h"
#ifndef DISABLE_TABS
#include "tab.h"
#endif /* !DISABLE_TABS */
#include "history.h"
#include "nav.h"
#include "doc.h"
#include "misc.h"
#include "commands.h"
#include "prefs.h"
#include "menu.h"
#include "capi.h"
#include "selection.h"

#include <gdk/gdkkeysyms.h>
#include "i18n.h"

/* FILE MENU */

/*
 * Create a new browser window
 */
void a_Commands_new_callback(GtkWidget *widget, gpointer client_data)
{
   BrowserWindow *nbw, *bw = (BrowserWindow *)client_data;

   nbw = a_Interface_browser_window_new(bw->main_window->allocation.width,
                                  bw->main_window->allocation.height, 0);
   /* focus the location bar */
   gtk_widget_grab_focus(nbw->location);
}

#ifndef DISABLE_TABS
/*
 * Create a new browser tab
 */
void a_Commands_new_tab_callback(GtkWidget *widget, gpointer client_data)
{
  BrowserWindow *bw = (BrowserWindow *) client_data;
  a_Tab_new(bw);
}

#endif
/*
 * Create and show the "Open file" dialog
 */
void a_Commands_openfile_callback(GtkWidget *widget, gpointer client_data)
{
   BrowserWindow *bw = (BrowserWindow *) client_data;

   a_Interface_openfile_dialog(bw);
}

/*
 * Create and show the "Open Url" dialog window
 *  OR (when prefs.popup_dialogs = FALSE)
 * focus location bar and select contents
 */
void a_Commands_openurl_callback(GtkWidget *widget, gpointer client_data)
{
   BrowserWindow *bw = (BrowserWindow *) client_data;
   if (prefs.popup_dialogs)
      a_Interface_open_dialog(widget, bw);
   else
      gtk_widget_grab_focus(bw->location);
}

/*
 * Show Configurarion Script
 */
void a_Commands_prefs_callback(GtkWidget *widget, gpointer client_data)
{
    gchar *cmd = g_strdup_printf("dillocfg&\n");
    system(cmd);
    g_free(cmd);
}

/*
 * Close browser window, and exit dillo if it's the last one.
 */
void a_Commands_close_callback(GtkWidget *widget, gpointer client_data)
{
   BrowserWindow *bw = (BrowserWindow *)client_data;
   a_Interface_findtext_dialog_close();
   gtk_widget_destroy(bw->main_window);
}

#ifndef DISABLE_TABS
/*
 * Close current browser tab, and exit dillo if it's the last one.
 */
void a_Commands_close_current_tab_callback(GtkWidget *widget, gpointer client_data)
{
   BrowserWindow *bw = (BrowserWindow *)client_data;
   a_Tab_close(bw->dd);
}

/*
 * Close a browser tab, and exit dillo if it's the last one.
 *
 * The difference with a_Commands_close_current_tab_callback is that this
 * callback is called with a specific dd as parameter, while the former is
 * called with only a bw
 */
void a_Commands_close_tab_callback(GtkWidget *widget, gpointer client_data)
{
   DilloDoc *dd = (DilloDoc *)client_data;
   a_Tab_close(dd);
}

#endif /* DISABLE_TABS */
/*
 * Free memory and quit dillo
 */
void a_Commands_exit_callback(GtkWidget *widget, gpointer client_data)
{
   a_Interface_quit_all();
}


/* PAGE MENU */

/*
 * Show current root document's source
 */
void a_Commands_viewsource_callback(GtkWidget *widget, gpointer client_data)
{
   BrowserWindow *bw = (BrowserWindow *) client_data;
   a_Commands_viewsource_doc_callback(widget, (gpointer) bw->dd);
}

/*
 * Show document's source code.
 */
void a_Commands_viewsource_doc_callback (GtkWidget *widget, gpointer client_data)
{
   DilloDoc *dd = (DilloDoc *)client_data;
   gchar *title, *buf;
   gint buf_size;
  
   /* Get page source data */
   a_Capi_get_buf(a_History_get_url(NAV_TOP(dd)), &buf, &buf_size);
   /* Show it */
   title = a_I18n_locale_to_DILLO_CHARSET(_("View Source"));
   a_Interface_text_window (&dd->bw->viewsource_window,
                            title, "view_source",
                            buf, buf_size,
                            530, 500);
   g_free(title);
}

/*
 * Show the detected HTML errors in current page.
 */
void a_Commands_view_page_bugs_callback (GtkWidget *widget,
                                         gpointer client_data)
{
   DilloHtmlLB *html_lb = client_data;
   gchar *title;

   title = a_I18n_locale_to_DILLO_CHARSET(_("Detected HTML errors"));
   a_Interface_text_window (&html_lb->dd->bw->pagebugs_window,
                            title, "page_bugs",
                            html_lb->page_bugs->str,
                            html_lb->page_bugs->len,
                            530, 500);
   g_free(title);
}

/*
 * ?
 */
void a_Commands_selectall_callback(GtkWidget *widget, gpointer client_data)
{
}

/*
 * Create and show the "Find Text" dialog window for this page
 */
void a_Commands_findtext_callback(GtkWidget *widget, gpointer client_data)
{
   BrowserWindow *bw = (BrowserWindow *) client_data;

   a_Interface_findtext_dialog( bw );
}

/*
 * Print the page!
 * ('cat page.html | html2ps | lpr -Pcool'   Why bother?  I think providing
 * such an option in a configurable way should cut it  --Jcid)
 */
void a_Commands_print_callback(GtkWidget *widget, gpointer client_data)
{
}

/* DOCUMENT (FRAME/IFRAME) MENU */
/*
 * Create and show the "Find Text" dialog window for this document
 */
void a_Commands_findtext_doc_callback(GtkWidget *widget, gpointer client_data)
{
   DilloDoc *dd = (DilloDoc *) client_data;

   /* set search list to this document's visible children */
   dd->bw->dd_list = a_Doc_get_visible_children(dd);
   a_Interface_findtext_dialog( dd->bw );
}

/*
 * Bring up the save doc dialog
 */
void a_Commands_save_doc_callback(GtkWidget *widget, gpointer client_data)
{
   DilloDoc *dd = (DilloDoc *) client_data;

   a_Menu_popup_set_url(dd->bw, a_History_get_url(NAV_TOP(dd)));
   a_Interface_save_link_dialog(widget, dd->bw);
}

/*
 * Show only this doc in the page
 */
void a_Commands_showonly_doc_callback(GtkWidget *widget, gpointer client_data)
{
   DilloDoc *dd = (DilloDoc *) client_data;

   a_Nav_push(dd->bw->dd, a_History_get_url(NAV_TOP(dd)));
}

/* TOOLBAR MENU */
/*
 * Abort all active connections for this page
 * (Downloads MUST keep flowing)
 */
void a_Commands_stop_callback(GtkWidget *widget, gpointer client_data)
{
   BrowserWindow *bw = client_data;
   gchar *msg_enc;
   
   a_Nav_cancel_expect(bw->dd);
   a_Doc_stop(bw->dd);
   a_Interface_set_button_sens(bw);
   msg_enc= a_I18n_locale_to_DILLO_CHARSET(_("Stopped"));
   a_Interface_msg(bw, msg_enc);
   g_free(msg_enc);
}

/*
 *  Back to previous page
 */
void a_Commands_back_callback(GtkWidget *widget, gpointer client_data)
{
   BrowserWindow *bw = (BrowserWindow *) client_data;

   a_Nav_back(bw->dd);
}

/*
 * Creates the menu for the forward navigation button.
 */
GtkMenu *a_Commands_forw_button_menu_creator_callback(GtkExtButton *button,
                                                      gpointer client_data)
{
   BrowserWindow *bw = client_data;

   if (bw->menu_popup.over_forw)
      gtk_widget_destroy(bw->menu_popup.over_forw);
   bw->menu_popup.over_forw = a_Menu_popup_history_new(bw, +1);
   return GTK_MENU(bw->menu_popup.over_forw);
}

/*
 * Creates the menu for the backward navigation button.
 */
GtkMenu *a_Commands_back_button_menu_creator_callback(GtkExtButton *button,
                                                      gpointer client_data)
{
   BrowserWindow *bw = client_data;

   if (bw->menu_popup.over_back)
      gtk_widget_destroy(bw->menu_popup.over_back);
   bw->menu_popup.over_back = a_Menu_popup_history_new(bw, -1);
   return GTK_MENU(bw->menu_popup.over_back);
}

/*
 * Handler for menu entries in the history menus. This one is called, when
 * the link should be opened in the same window.
 */
void a_Commands_history_callback_same_bw(GtkWidget *widget,
                                         gpointer client_data)
{
   BrowserWindow *bw = client_data;

   a_Nav_jump_callback(widget, bw->dd, 0);
}

/*
 * Handler for menu entries in the history menus. This one is called, when
 * the link should be opened in a new window.
 */
void a_Commands_history_callback_new_bw(GtkWidget *widget,
                                        gpointer client_data)
{
   BrowserWindow *bw = client_data;

   a_Nav_jump_callback(widget, bw->dd, 1);
}

/*
 * Handler for menu entries in the history menus. This one is called, when
 * the link should be opened in a new tab.
 */
void a_Commands_history_callback_new_tab(GtkWidget *widget,
                                        gpointer client_data)
{
   BrowserWindow *bw = client_data;

   a_Nav_jump_callback(widget, bw->dd, 2);
}

/*
 * Go to the next page in the history buffer
 */
void a_Commands_forw_callback(GtkWidget *widget, gpointer client_data)
{
   BrowserWindow *bw = (BrowserWindow *) client_data;

   a_Nav_forw(bw->dd);
}

/*
 * Start the reload process for a document
 */
void a_Commands_reload_doc_callback(GtkWidget *widget, gpointer client_data)
{
   DilloDoc *dd = (DilloDoc *)client_data;

   a_Nav_reload(dd);
}

/*
 * Start the reload process for the page
 */
void a_Commands_reload_callback(GtkWidget *widget, gpointer client_data)
{
   BrowserWindow *bw = (BrowserWindow *) client_data;

   a_Interface_findtext_dialog_close();
   a_Nav_reload(bw->dd);
}

/*
 * Go home!
 */
void a_Commands_home_callback(GtkWidget *widget, gpointer client_data)
{
   BrowserWindow *bw = (BrowserWindow *) client_data;

   a_Nav_home(bw->dd);
}

/*
 * Open home with new window/tab
 */
void a_Commands_home_nw_callback(GtkWidget *widget, gpointer client_data)
{
   BrowserWindow *bw = (BrowserWindow *) client_data;
   DilloDoc *newdd;

#ifndef DISABLE_TABS
   if (prefs.tab_instead_of_window) {
      newdd = a_Doc_new();
      a_Tab_add(bw, newdd);
      a_Tab_switch(newdd);
      a_Nav_home(newdd);
      return;
   }
#endif
   {
      gint width, height;
      BrowserWindow *newbw;

      gdk_window_get_size (bw->main_window->window,
               &width, &height);
      newbw = a_Interface_browser_window_new(width, height, 0);
      newdd = newbw->dd;
      a_Nav_home(newdd);
   }
}

/*
 * Bring up the save page dialog
 */
void a_Commands_save_callback(GtkWidget *widget, gpointer client_data)
{
   BrowserWindow *bw = (BrowserWindow *) client_data;  

   a_Interface_save_dialog(widget, bw);
}


/*
 * Bring up the save link dialog
 */
void a_Commands_save_link_callback(GtkWidget *widget, gpointer client_data)
{
   BrowserWindow *bw = (BrowserWindow *) client_data;

   a_Interface_save_link_dialog(widget, bw);
}

#ifndef DISABLE_TABS
/* VIEW MENU */
void a_Commands_tab_prev_callback(GtkWidget *widget, gpointer client_data)
{
  BrowserWindow *bw = (BrowserWindow *) client_data;
  gtk_notebook_prev_page(GTK_NOTEBOOK(bw->container));
}

void a_Commands_tab_next_callback(GtkWidget *widget, gpointer client_data)
{
  BrowserWindow *bw = (BrowserWindow *) client_data;
  gtk_notebook_next_page(GTK_NOTEBOOK(bw->container));
}

#endif
/* BOOKMARKS MENU */
/*
 * Add a bookmark for the current doc to the current bookmark widget.
 */
void a_Commands_addbm_callback(GtkWidget *widget, gpointer client_data)
{
   BrowserWindow *bw = (BrowserWindow *) client_data;

   a_Bookmarks_add(bw->dd, a_History_get_url(NAV_TOP(bw->dd)));
}

void a_Commands_addbm_doc_callback(GtkWidget *widget, gpointer client_data)
{
   DilloDoc *dd = (DilloDoc *) client_data;

   a_Bookmarks_add(dd, a_History_get_url(NAV_TOP(dd)));
}

/*
 * add a bookmark for the current link
 */
void a_Commands_addbm_link_callback(GtkWidget *widget, gpointer client_data)
{
   BrowserWindow *bw = (BrowserWindow *) client_data;

   a_Bookmarks_add(bw->dd, a_Menu_popup_get_url(bw));
}

/*
 * Show the bookmarks-file as rendered html
 */
void a_Commands_viewbm_callback(GtkWidget *widget, gpointer client_data)
{
   BrowserWindow *bw = (BrowserWindow *) client_data;

   a_Bookmarks_show(bw->dd);
}

/*
 * Open bookmarks with new window/tab
 */
void a_Commands_viewbm_nw_callback(GtkWidget *widget, gpointer client_data)
{
   BrowserWindow *bw = (BrowserWindow *) client_data;
   DilloDoc *newdd;

#ifndef DISABLE_TABS
   if (prefs.tab_instead_of_window) {
      newdd = a_Doc_new();
      a_Tab_add(bw, newdd);
      a_Tab_switch(newdd);
      a_Bookmarks_show(newdd);
      return;
   }
#endif
   {
      gint width, height;
      BrowserWindow *newbw;

      gdk_window_get_size (bw->main_window->window,
               &width, &height);
      newbw = a_Interface_browser_window_new(width, height, 0);
      newdd = newbw->dd;
      a_Bookmarks_show(newdd);
   }
}


/* HELP MENU */

/*
 * This one was intended as a link to help-info on the web site, but
 * currently points to the home page  --Jcid
 */
void a_Commands_helphome_callback(GtkWidget *widget, gpointer client_data)
{
   BrowserWindow *bw = (BrowserWindow *) client_data;
   DilloUrl *url = a_Url_new(DILLO_HOME, NULL, 0, 0, 0);

   a_Nav_push(bw->dd, url);
   a_Url_free(url);
}


/* RIGHT BUTTON POP-UP MENU */

/*
 * Open link in browser-window
 */
void a_Commands_open_link_callback(GtkWidget *widget, gpointer client_data)
{
   BrowserWindow *bw = (BrowserWindow *)client_data;

   a_Nav_push(bw->dd, a_Menu_popup_get_url(bw));
}

/*
 * Open link in another browser-window
 */
void a_Commands_open_link_nw_callback(GtkWidget *widget, gpointer client_data)
{
   BrowserWindow *bw = (BrowserWindow *)client_data;
   gint width, height;
   BrowserWindow *newbw;

   gdk_window_get_size (bw->main_window->window, &width, &height);
   newbw = a_Interface_browser_window_new(width, height, 0);
   a_Nav_push(newbw->dd, a_Menu_popup_get_url(bw));
}

/*
 * Duplicate current page in a new window
 */
void a_Commands_duplicate_callback(GtkWidget *widget, gpointer client_data)
{
   BrowserWindow *bw = (BrowserWindow *)client_data;
   if(bw->dd->nav_stack == NULL) {
       a_Commands_new_callback(widget, client_data);
       return;
   }

   a_Menu_popup_set_url(bw, a_History_get_url(NAV_TOP(bw->dd)));
   a_Commands_open_link_nw_callback(widget, client_data);
}

#ifndef DISABLE_TABS
/*
 * Open link in another tab
 */
void a_Commands_open_link_nw_tab_callback(GtkWidget *widget, gpointer client_data)
{
   BrowserWindow *bw = (BrowserWindow *)client_data;
   DilloDoc *newdd;

   newdd = a_Doc_new();
   a_Tab_add(bw, newdd);

   if (!prefs.tab_load_in_background)
      a_Tab_switch(newdd);

   a_Nav_push(newdd, a_Menu_popup_get_url(bw));
}

/*
 * Duplicate current page in a new tab
 */
void a_Commands_duplicate_tab_callback(GtkWidget *widget, gpointer client_data)
{
   BrowserWindow *bw = (BrowserWindow *)client_data;
   if(bw->dd->nav_stack == NULL) {
       a_Commands_new_tab_callback(widget, client_data);
       return;
   }

   a_Menu_popup_set_url(bw, a_History_get_url(NAV_TOP(bw->dd)));
   a_Commands_open_link_nw_tab_callback(widget, client_data);
}
#endif

/*
 * Called when the user wants the popup's URL for pasting.
 */
void a_Commands_select_popup_url_callback(GtkWidget *widget, gpointer data)
{
   BrowserWindow *bw = (BrowserWindow *)data;
   a_Selection_set_selection(widget, URL_STR(a_Menu_popup_get_url(bw)));
}

/* BUG METER POP-UP MENU */

/*
 * Helper function for validation.
 */
static void Commands_ob_validate_page(BrowserWindow *bw, const gchar *fmt)
{
   GString *gstr = g_string_sized_new(128);
   gchar *o_url, *e_url, *n_url;
   DilloUrl *url;

   /* prepare the validation request URI */
   o_url = URL_STR(a_History_get_url(NAV_TOP(bw->dd)));
   e_url = a_Misc_escape_chars(o_url, ":/?");
   n_url = e_url ? e_url : o_url;
   g_string_sprintf(gstr, fmt, n_url);
   if (n_url != o_url)
      g_free(n_url);
   url = a_Url_new(gstr->str, NULL, 0, 0, 0);
   g_string_free(gstr, TRUE);

   a_Nav_push(bw->dd, url);
   a_Url_free(url);
}

/*
 * Validate current page with the W3C's validator.
 */
void a_Commands_ob_w3c_callback(GtkWidget *widget, gpointer client_data)
{
   BrowserWindow *bw = (BrowserWindow *)client_data;
   const gchar *w3c_fmt="http://validator.w3.org/check?uri=%s";

   Commands_ob_validate_page(bw, w3c_fmt);
}

/*
 * Validate current page with the WDG's validator.
 */
void a_Commands_ob_wdg_callback(GtkWidget *widget, gpointer client_data)
{
   BrowserWindow *bw = (BrowserWindow *)client_data;
   const gchar *wdg_fmt=
      "http://www.htmlhelp.org/cgi-bin/validate.cgi?url=%s&warnings=yes";

   Commands_ob_validate_page(bw, wdg_fmt);
}

/*
 * Request the info page for bug meter.
 */
void a_Commands_ob_info_callback(GtkWidget *widget, gpointer client_data)
{
   BrowserWindow *bw = (BrowserWindow *)client_data;
   const gchar *info_str = "http://www.dillo.org/help/bug_meter.html";
   DilloUrl *url = a_Url_new(info_str, NULL, 0, 0, 0);

   /* here we should un-highlight the selected text */
   a_Nav_push(bw->dd, url);
   a_Url_free(url);
}

/* MAIN WINDOW */

/*
 * callback for full screen toggle
 */
void a_Commands_full_screen_callback (gpointer client_data)
{
   BrowserWindow *bw = (BrowserWindow *) client_data;
   a_Interface_toggle_panel(bw, TRUE);
}

/*
 * Handler for double-mouse-clicks that don't belong to the viewport.
 */
gint a_Commands_click_callback(gpointer client_data, GdkEventButton *event)
{
   BrowserWindow *bw = (BrowserWindow *) client_data;
   if (event->type == GDK_2BUTTON_PRESS && event->button == 1)
      a_Interface_toggle_panel(bw, TRUE);
   return TRUE;
}

/*
 * Handler for key presses that don't belong to the viewport.
 * (Used to customize the interface a bit)
 */
void a_Commands_key_press_handler(GtkWidget *widget,
                                  GdkEventKey *event,
                                  gpointer client_data)
{
   BrowserWindow *bw = client_data;
   switch (event->keyval) {
   case GDK_BackSpace:
     /* This key is handled here because GTK accel group ignores it */
     if (event->state & GDK_SHIFT_MASK)
       a_Commands_forw_callback(NULL, bw);
     else
       a_Commands_back_callback(NULL, bw);
     break;
   case GDK_slash:
     /* This key is handled here because GTK accel group ignores it */
     a_Commands_findtext_callback(NULL, bw);
     break;
   case GDK_Escape:
     a_Interface_findtext_dialog_close();
   default:
     /* g_print(">> Key pressed!\n"); */
     break;
   }
}
