/*
 * File: i18n_gtk.c
 *
 * Copyright (C) 2003 Kiyo <kiyo@teki.jpn.ph>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 */
/* gtk+ UTF-8 wrapper. */
#include <string.h>
#include <gtk/gtk.h>
#include "i18n_gtk.h"
#include "i18n.h"

#ifndef DISABLE_I18N
GtkWidget*  a_I18n_gtk_label_new (const gchar *str) {
    gchar *locale_str = a_I18n_locale_from_DILLO_CHARSET(str, -1);
    GtkWidget* widget = gtk_label_new(locale_str);
    g_free(locale_str);
    return widget;
}

void a_I18n_gtk_label_set_text(GtkLabel *label, const gchar *str) {
    gchar *locale_str = a_I18n_locale_from_DILLO_CHARSET(str, -1);
    gtk_label_set_text(label, locale_str);
    g_free(locale_str);
}

void a_I18n_gtk_window_set_title(GtkWindow *window, const gchar *title) {
    gchar *locale_str = a_I18n_locale_from_DILLO_CHARSET(title, -1);
    gtk_window_set_title(window, locale_str);
    g_free(locale_str);
}

gchar* a_I18n_gtk_editable_get_chars(GtkEditable *editable,
					gint start_pos, gint end_pos) {
    gchar *locale_str = gtk_editable_get_chars(editable, start_pos, end_pos);
    gchar *utf8_str = a_I18n_locale_to_DILLO_CHARSET(locale_str);
    g_free(locale_str);
    return utf8_str;
}

void a_I18n_gtk_editable_insert_text(GtkEditable *editable,
					const gchar *new_text, gint new_text_length,
					gint *position) {
    gchar *locale_str = a_I18n_locale_from_DILLO_CHARSET(new_text,
            new_text_length);
    gtk_editable_insert_text(editable,
            locale_str, (!locale_str) ? 0 : strlen(locale_str), position);
    g_free(locale_str);
}

GtkWidget* a_I18n_gtk_radio_menu_item_new_with_label(GSList *group,
        const gchar *label) {
    gchar *locale_str = a_I18n_locale_from_DILLO_CHARSET(label, -1);
    GtkWidget* widget = gtk_radio_menu_item_new_with_label(group, locale_str);
    g_free(locale_str);
    return widget;
}

GtkWidget* a_I18n_gtk_list_item_new_with_label(const gchar *label) {
    gchar *locale_str = a_I18n_locale_from_DILLO_CHARSET(label, -1);
    GtkWidget* widget = gtk_list_item_new_with_label(locale_str);
    g_free(locale_str);
    return widget;
}

void a_I18n_gtk_text_insert(GtkText *text, GdkFont *font,
				     GdkColor *fore, GdkColor *back,
				     const char *chars, gint length) {
    gchar *locale_str = a_I18n_locale_from_DILLO_CHARSET(chars, length);
    gtk_text_insert(text, font, fore, back, locale_str,
            (!locale_str) ? 0 : strlen(locale_str));
    g_free(locale_str);
}

GtkWidget* a_I18n_gtk_button_new_with_label(const gchar *label) {
    gchar *locale_str = a_I18n_locale_from_DILLO_CHARSET(label, -1);
    GtkWidget* widget = gtk_button_new_with_label(locale_str);
    g_free(locale_str);
    return widget;
}

void a_I18n_gtk_entry_set_text(GtkEntry *entry, const gchar *text) {
    gchar *locale_str = a_I18n_locale_from_DILLO_CHARSET(text, -1);
    gtk_entry_set_text(entry, (locale_str)? locale_str : "");
    g_free(locale_str);
}

gchar* a_I18n_gtk_entry_get_text(GtkEntry *entry) {
    gchar *locale_str = gtk_entry_get_text(entry);
    gchar *utf8_str = a_I18n_locale_to_DILLO_CHARSET(locale_str);
    return utf8_str;
}

GtkWidget* a_I18n_gtk_menu_item_new_with_label(const gchar *label) {
    gchar *locale_str = a_I18n_locale_from_DILLO_CHARSET(label, -1);
    GtkWidget* widget = gtk_menu_item_new_with_label(locale_str);
    g_free(locale_str);
    return widget;
}

void a_I18n_gdk_draw_string (GdkDrawable *drawable,
        GdkFont *font,
        GdkGC *gc,
        gint x,
        gint y,
        const gchar *string) {
    gchar *locale_str = a_I18n_locale_from_DILLO_CHARSET(string, -1);
    gdk_draw_string (drawable, font, gc, x, y, locale_str);   
    g_free(locale_str);
}

gint a_I18n_gdk_text_width (GdkFont *font,
        const gchar *text, gint text_length) {
    gchar *locale_str = a_I18n_locale_from_DILLO_CHARSET(text,
            text_length);
    gint len;
    if (!locale_str) return 0;
    len = gdk_text_width(font, locale_str, strlen(locale_str));
    g_free(locale_str);
    return len;
}

#endif /* DISABLE_I18n */

